/// <reference path="lib/ts/phaser.d.ts" />
/// <reference path="lib/ts/spriterRuntime.d.ts" />
/// <reference path="spriterObj.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Dog = (function (_super) {
    __extends(Dog, _super);
    function Dog(game, dogData, x, y) {
        _super.call(this, "dog", "dog", "idle", game, dogData, x, y);
        this.DOGSPEED = 4;
        this.DOGSPEEDRUN = 8;
        this.barking = false;
        this.lastMove = 0;
        this.movingSince = 0;
        this.oldSpeed = new Phaser.Point(0, 0);
        this.spaceWasDown = false;
        this.spriterGroup.setListener(this);
        this.inverted = -1;
        this.game = game;
    }
    Dog.prototype.onAnimationFinished = function (aSpriterGroup) {
        if (aSpriterGroup.getCurrentAnimationName() == "sitDown") {
            aSpriterGroup.setAnimationByName("sitIdle");
        }
        if (aSpriterGroup.getCurrentAnimationName() == "standUp") {
            aSpriterGroup.setAnimationByName("idle");
        }
        if (aSpriterGroup.getCurrentAnimationName() == "bark") {
            aSpriterGroup.setAnimationByName("idle");
            this.barking = false;
        }
    };
    Dog.prototype.isBarking = function () {
        return this.barking;
    };
    Dog.prototype.isRunning = function (tick) {
        return this.movingSince > 0 && tick - this.movingSince > 70;
    };
    Dog.prototype.calcAnimation = function (tick) {
        //noinspection JSSuspiciousNameCombination
        if (Math.abs(this.speed.x) > 0.01 || Math.abs(this.speed.y) > 0.01) {
            if (this.movingSince == 0) {
                this.movingSince = tick;
            }
            if (this.isRunning(tick)) {
                this.setAnimation("run");
            }
            else if (this.spriterGroup.getCurrentAnimationName() == "run") {
                this.setAnimation("walk");
            }
            this.lastMove = tick;
            if (this.spriterGroup.getCurrentAnimationName() == "idle") {
                this.setAnimation("walk");
            }
            else if (this.spriterGroup.getCurrentAnimationName() == "sitIdle") {
                this.setAnimation("standUp");
            }
        }
        else {
            if (this.spriterGroup.getCurrentAnimationName() == "walk" || this.spriterGroup.getCurrentAnimationName() == "run") {
                this.setAnimation("idle");
            }
            if (tick > this.lastMove + 60 && this.spriterGroup.getCurrentAnimationName() != "sitIdle" && this.spriterGroup.getCurrentAnimationName() != "bark") {
                this.setAnimation("sitDown");
            }
        }
    };
    Dog.prototype.control = function (tick, sound) {
        this.speed.set(0, 0);
        if (this.game.input.keyboard.isDown(Phaser.Keyboard.SPACEBAR)) {
            if (!this.spaceWasDown) {
                if (!(this.spriterGroup.getCurrentAnimationName() == 'bark')) {
                    this.spriterGroup.setAnimationByName('bark');
                    var rnd = Math.random();
                    if (rnd > 0.66) {
                        sound.play('bark1');
                    }
                    else if (rnd > 0.33) {
                        sound.play('bark2');
                    }
                    else {
                        sound.play('bark3');
                    }
                    this.barking = true;
                }
            }
        }
        else {
            this.spaceWasDown = false;
        }
        if (this.spriterGroup.getCurrentAnimationName() == 'bark') {
        }
        else {
            if (this.game.input.keyboard.isDown(Phaser.Keyboard.LEFT)) {
                this.speed.add(-1, 0);
            }
            if (this.game.input.keyboard.isDown(Phaser.Keyboard.RIGHT)) {
                this.speed.add(1, 0);
            }
            if (this.game.input.keyboard.isDown(Phaser.Keyboard.UP)) {
                this.speed.add(0, -1);
            }
            if (this.game.input.keyboard.isDown(Phaser.Keyboard.DOWN)) {
                this.speed.add(0, 1);
            }
            if (this.speed.x == 0 && this.speed.y == 0) {
                this.movingSince = 0;
            }
            this.oldSpeed.set(this.speed.x, this.speed.y);
            this.speed.normalize();
            if (this.isRunning(tick)) {
                this.speed.multiply(this.DOGSPEEDRUN, this.DOGSPEEDRUN);
            }
            else {
                this.speed.multiply(this.DOGSPEED, this.DOGSPEED);
            }
        }
    };
    return Dog;
})(SpriterObj);
//# sourceMappingURL=Dog.js.map