/// <reference path="lib/ts/phaser.d.ts" />
/// <reference path="lib/ts/spriterRuntime.d.ts" />
/// <reference path="spriterObj.ts" />


class Dog extends SpriterObj implements Spriter.ISpriterGroupListener{


    onAnimationFinished(aSpriterGroup:Spriter.SpriterGroup):void {
        if (aSpriterGroup.getCurrentAnimationName() == "sitDown") {
            aSpriterGroup.setAnimationByName("sitIdle");
        }

        if (aSpriterGroup.getCurrentAnimationName() == "standUp") {
            aSpriterGroup.setAnimationByName("idle");
        }

        if (aSpriterGroup.getCurrentAnimationName() == "bark") {
            aSpriterGroup.setAnimationByName("idle");
            this.barking = false;
        }
    }

    DOGSPEED:number = 4;
    DOGSPEEDRUN:number = 8;

    game:Phaser.Game;

    barking:boolean = false;

    isBarking() {
        return this.barking;
    }

    constructor(game:Phaser.Game, dogData:any, x:number, y:number) {
        super("dog", "dog", "idle", game, dogData, x, y);
        this.spriterGroup.setListener(this);
        this.inverted = -1;
        this.game = game;
    }


    lastMove:number = 0;
    movingSince: number = 0;

    isRunning(tick:number) {
        return this.movingSince > 0 && tick - this.movingSince > 70;
    }

    calcAnimation(tick: number) {
        //noinspection JSSuspiciousNameCombination
        if (Math.abs(this.speed.x) > 0.01 || Math.abs(this.speed.y) > 0.01) {
            if (this.movingSince == 0) {
                this.movingSince = tick;
            }

            if (this.isRunning(tick)) {
                this.setAnimation("run");
            } else if (this.spriterGroup.getCurrentAnimationName() == "run"){
                this.setAnimation("walk");
            }

            this.lastMove = tick;
            if (this.spriterGroup.getCurrentAnimationName() == "idle") {
                this.setAnimation("walk");
            } else if (this.spriterGroup.getCurrentAnimationName() == "sitIdle") {
                this.setAnimation("standUp");
            }
        } else {
            if (this.spriterGroup.getCurrentAnimationName() == "walk" || this.spriterGroup.getCurrentAnimationName() == "run" ) {
                this.setAnimation("idle");
            }
            if (tick > this.lastMove + 60 && this.spriterGroup.getCurrentAnimationName() != "sitIdle" && this.spriterGroup.getCurrentAnimationName() != "bark") {
                this.setAnimation("sitDown");
            }
        }
    }

    oldSpeed:Phaser.Point = new Phaser.Point(0, 0);
    spaceWasDown:boolean = false;

    control(tick:number, sound:Phaser.Sound) {
        this.speed.set(0, 0);

        if (this.game.input.keyboard.isDown(Phaser.Keyboard.SPACEBAR)) {
            if (!this.spaceWasDown) {
                if (!(this.spriterGroup.getCurrentAnimationName() == 'bark')) {
                    this.spriterGroup.setAnimationByName('bark');
                    var rnd = Math.random();
                    if (rnd > 0.66) {
                        sound.play('bark1');
                    } else if (rnd > 0.33) {
                        sound.play('bark2');
                    } else {
                        sound.play('bark3');
                    }
                    this.barking = true;
                }
            }
        } else {
            this.spaceWasDown = false;
        }

        if (this.spriterGroup.getCurrentAnimationName() == 'bark') {
            //barking right now
        } else {

            if (this.game.input.keyboard.isDown(Phaser.Keyboard.LEFT)) {
                this.speed.add(-1, 0);
            }
            if (this.game.input.keyboard.isDown(Phaser.Keyboard.RIGHT)) {
                this.speed.add(1, 0);
            }
            if (this.game.input.keyboard.isDown(Phaser.Keyboard.UP)) {
                this.speed.add(0, -1);
            }
            if (this.game.input.keyboard.isDown(Phaser.Keyboard.DOWN)) {
                this.speed.add(0, 1);
            }

            if (this.speed.x == 0 && this.speed.y == 0) {
                this.movingSince = 0;
            }

            this.oldSpeed.set(this.speed.x, this.speed.y);

            this.speed.normalize();
            if (this.isRunning(tick)) {
                this.speed.multiply(this.DOGSPEEDRUN, this.DOGSPEEDRUN);
            } else {
                this.speed.multiply(this.DOGSPEED, this.DOGSPEED);
            }
        }

    }

}