/*
Todo:

- Optional Schafe bewegen sich auch mal von alleine

 */


/// <reference path="lib/ts/phaser.d.ts" />
/// <reference path="lib/ts/spriterRuntime.d.ts" />
/// <reference path="lib/ts/jquery.d.ts" />
/// <reference path="sheep.ts" />
/// <reference path="Dog.ts" />
/// <reference path="spriterObj.ts" />
class Game {
    constructor() {
        this.game = new Phaser.Game(1280, 720, Phaser.AUTO, "game",
            { preload: this.preload, create: this.create, update: this.update, drawMinimap: this.drawMinimap,
              updateMinimapOverlay: this.updateMinimapOverlay, reset: this.reset, updateText: this.updateText,
              setupText: this.setupText, logscore: this.logscore
            });
    }

    game: Phaser.Game;
    logo: Phaser.Sprite;
    dog:Dog;
    sheeps:Sheep[];
    tick:number;
    map:Phaser.Tilemap;
    layer:Phaser.TilemapLayer;
    minimap:Phaser.Graphics;
    minimapOverlay:Phaser.Graphics;
    uiGroup:Phaser.Group;
    sheepCount:number;
    secondsLeft:number;
    scoreText:Phaser.Text;
    timeText:Phaser.Text;
    infoText:Phaser.Text;
    sound:Phaser.Sound;

    startButton:Phaser.Button;


    updateText() {
        this.scoreText.setText("herd size: " + this.sheepCount);
        this.timeText.setText("time remaining: " + this.secondsLeft + " seconds");
    }

    preload() {
        this.game.load.image("startButton", "assets/startButton.png");
        this.game.load.image("logo", "assets/title.png");
        this.game.load.xml("sheepDataXml", "assets/sheep.scml");
        this.game.load.atlas("sheep", "assets/sheep.png", "assets/sheep.json");
        this.game.load.atlas("sheep2", "assets/sheep2.png", "assets/sheep.json");
        this.game.load.xml("dogDataXml", "assets/dog.scml");
        this.game.load.atlas("dog", "assets/dog.png", "assets/dog.json");

        this.game.load.tilemap('tilemap', 'assets/map.json', null, Phaser.Tilemap.TILED_JSON);
        this.game.load.image('tiles', 'assets/tiles.png');

        this.game.load.audio('sfx', 'assets/sounds.mp3');
    }

    reset() {
        this.infoText.visible = false;
        this.sheepCount = 0;
        this.secondsLeft = 180;
        this.startButton.visible = false;
        this.logo.visible = false;

        var sheepsPlaced = 0;
        while (sheepsPlaced < 50) {
            var x = Math.random() * 60 * 128;
            var y = Math.random() * 60 * 128;
            if (!SpriterObj.isBlocked(x, y, this.map, this.layer)) {
                if (x < 14 * 128 && y < 14 * 128) {
                    continue;
                }
                this.sheeps[sheepsPlaced].setPos(x, y);
                this.sheeps[sheepsPlaced].counted = false;
                sheepsPlaced++;
            }
        }
        this.dog.setPos(300, 300);
    }

    create() {
        this.sheepCount = 0;

        this.sound = this.game.add.audio('sfx');
        this.sound.allowMultiple = true;

        this.sound.addMarker('ba1', 0.29, 1.306 - 0.29);
        this.sound.addMarker('ba2', 2.229, 3.042 - 2.229);
        this.sound.addMarker('ba3', 3.541, 4.4 - 3.541);

        this.sound.addMarker('bark1', 6.873, 7.233 - 6.873);
        this.sound.addMarker('bark2', 7.732, 8.382 - 7.732);
        this.sound.addMarker('bark3', 8.789, 9.694 -8.789);

        this.map = this.game.add.tilemap("tilemap");
        this.map.addTilesetImage("tiles", "tiles");
        this.layer = this.map.createLayer("land");

        this.uiGroup = this.game.add.group();

        this.sheeps = [];
        this.tick = 0;
        this.game.stage.backgroundColor = 0x88aa00;
        this.logo = this.game.add.sprite(this.game.width/2, 200, "logo");
        this.logo.anchor.setTo(0.5, 0.5);
        this.logo.fixedToCamera = true;

        var spriterLoader = new Spriter.Loader();
        var spriterFile = new Spriter.SpriterXml(this.game.cache.getXML("sheepDataXml"));
        var sheepData = spriterLoader.load(spriterFile);

        for (var i = 0; i < 50; i++) {
            this.sheeps.push(Sheep.createSheep(this.game, sheepData, 0, 0));
        }

        spriterFile = new Spriter.SpriterXml(this.game.cache.getXML("dogDataXml"));
        var dogData = spriterLoader.load(spriterFile);

        this.dog = new Dog(this.game, dogData, 300, 300);

        this.game.world.setBounds(0, 0, 60 * 128, 60 * 128);
        this.game.camera.follow(<any>this.dog.spriterGroup, Phaser.Camera.FOLLOW_TOPDOWN_TIGHT);

        this.minimap = this.game.add.graphics(this.game.width - 260, 40, this.uiGroup);
        this.drawMinimap(this.minimap, this.map);

        this.minimapOverlay = this.game.add.graphics(this.game.width - 260, 40, this.uiGroup);
        this.updateMinimapOverlay(this.minimapOverlay, this.sheeps, this.dog);

        this.minimap.fixedToCamera = true;
        this.minimapOverlay.fixedToCamera = true;

        this.scoreText = this.game.add.text(this.game.width - 240, 2, "herd size: 0", {}, this.uiGroup);
        this.setupText(this.scoreText, 14);

        this.timeText = this.game.add.text(this.game.width - 240, 20, "time remaining: 180 seconds", {}, this.uiGroup);
        this.setupText(this.timeText, 14);

        this.infoText = this.game.add.text(this.game.width/2, 600, "To begin playing click the start button\n control the dog with the cursor keys and the space bar\nto score points herd the sheep into the starting area (blue on the mini map)", {}, this.uiGroup);
        this.infoText.anchor.set(0.5, 0.5);
        this.setupText(this.infoText, 20);


        var button = this.game.add.button(this.game.width/2, this.game.height/2, "startButton", this.reset, this, null, null, null, null, this.uiGroup);
        button.anchor.set(0.5,0.5);
        button.fixedToCamera = true;
        this.startButton = button;


        this.secondsLeft = -1;
    }

    setupText(text, size) {
        text.font = "Arial Black";
        text.fontSize = size;
        text.fontWeight = "bold";
        text.stroke = "#000000";
        text.strokeThickness = 6;
        text.fill = "#FFFFFF";
        text.align = "center";
        text.fixedToCamera = true;
    }

    update() {
        if (this.secondsLeft < 1) {
            if (this.secondsLeft == 0) {

                //nochmal schnell die schafe zählen
                for (var i = 0; i < this.sheeps.length; i++) {
                    if (this.sheeps[i].counted == false && this.sheeps[i].getPos().x <  13 * 128 && this.sheeps[i].getPos().y < 14 * 128 ) {
                        this.sheeps[i].counted = true;
                        this.sheepCount++;
                    }
                }


                this.secondsLeft = -1;
                this.infoText.visible = true;
                this.logo.visible = true;
                this.infoText.setText("The time is up - thanks for playing \"grow the herd\" \nYour final score: " + this.sheepCount + "\n Click start to play again");
                this.logscore();
                this.startButton.visible = true;
            }
            return;
        }

        this.tick++;

        if (this.tick % 10 == 0) {
            this.updateMinimapOverlay(this.minimapOverlay, this.sheeps, this.dog);
        }

        if (this.tick % 60 == 0) {
            this.secondsLeft--;
            this.updateText();
        }

        if (this.tick % 30 == 0) {
            for (var i = 0; i < this.sheeps.length; i++) {
                if (this.sheeps[i].counted == false && this.sheeps[i].getPos().x <  13 * 128 && this.sheeps[i].getPos().y < 14 * 128 ) {
                    this.sheeps[i].counted = true;
                    this.sheepCount++;
                }
            }
        }

        this.dog.control(this.tick, this.sound);
        this.dog.update(this.tick, this.map, this.layer);
        this.dog.updateAnimation();

        for (var i = 0; i < this.sheeps.length; i++) {
            this.sheeps[i].updateAnimation();
            this.sheeps[i].updateAI(this.tick, this.sheeps, this.dog, this.sound);
            this.sheeps[i].update(this.tick, this.map, this.layer);
        }

        this.game.world.sort("y", Phaser.Group.SORT_ASCENDING);
        this.game.world.bringToTop(this.uiGroup);
    }



    drawMinimap(minimap:Phaser.Graphics, map:Phaser.Tilemap) {
        minimap.lineStyle(2, 0xC87137, 1);
        minimap.beginFill(0x6E8900, 1);
        minimap.drawRect(0, 0, 242, 242);
        minimap.endFill();
        minimap.lineStyle(0);
        for (var y = 0; y < 60; y++) {
            for (var x = 0; x < 60; x++) {
                var index = map.getTile(x, y).index;
                if (index  > 11 && index != 27) {
                    minimap.beginFill(0x824824, 1);
                    minimap.drawRect(1 + x * 4, 1 + y * 4, 4, 4);
                    minimap.endFill();
                }
            }
        }

        minimap.lineStyle(1, 0x00A9B2, 1);
        minimap.beginFill(0x00A9B2, 0.5);
        minimap.drawRect(4, 4, 11 * 4, 12 * 4);
        minimap.endFill();
        minimap.lineStyle(0);
    }

    private updateMinimapOverlay(overlay:Phaser.Graphics, sheeps:Sheep[], dog:Dog) {
        overlay.clear();
        overlay.beginFill(0xFF0000, 1);
        overlay.drawRect(dog.getPos().x/128 * 4, -2 + dog.getPos().y/128 * 4, 4, 4);
        overlay.endFill();
        for (var i = 0; i < sheeps.length; i++) {
            overlay.beginFill(0xB3B5AA, 1);
            overlay.drawRect(sheeps[i].getPos().x/128 * 4, -2 + sheeps[i].getPos().y/128 * 4, 4, 4);
            overlay.endFill();
        }
    }

    logscore() {
        if (this.sheepCount == 0)
            return;
        $.ajax({
            type: "GET",
            url: "/log.php",
            data: {gameid: 8, score: this.sheepCount, level: "-", date: new Date}
        });
    }

}

window.onload = () => {
    new Game();
};
