/// <reference path="phaser.d.ts" />

declare module Spriter {
    class Spriter {
        private _folders;
        private _entities;
        constructor();
        addFolder(aFolder: Folder): void;
        getFolderById(aId: number): Folder;
        getFolderByName(aName: string): Folder;
        addEntity(aEntitiy: Entity): void;
        getEntityById(aId: number): Entity;
        getEntityByName(aName: string): Entity;
    }
	   class SpriterBone {
        static UPDATE_X: number;
        static UPDATE_Y: number;
        static UPDATE_SCALE_X: number;
        static UPDATE_SCALE_Y: number;
        static UPDATE_PIVOT_X: number;
        static UPDATE_PIVOT_Y: number;
        static UPDATE_ANGLE: number;
        static UPDATE_ALPHA: number;
        private _on;
        parent: number;
        timelineKey: number;
        key: SpatialTimelineKey;
        timeFrom: number;
        timeTo: number;
        from: SpatialInfo;
        to: SpatialInfo;
        transformed: SpatialInfo;
        updateMask: number;
        constructor();
        setOn(aOn: boolean): void;
        getOn(): boolean;
        setKey(aAnimation: Animation, aTimelineId: number, aKeyId: number): void;
        tween(aTime: number): void;
        update(aParent: SpatialInfo): void;
        private scalePosition(aParentScaleX, aParentScaleY);
        private rotatePosition(aParentAngle);
        private translatePosition(aParentX, aParentY);
        private getTweenTime(aTime);
        private linear(aA, aB, aT);
        private quadratic(aA, aB, aC, aT);
        private cubic(aA, aB, aC, aD, aT);
        private angleLinear(aAngleA, aAngleB, aSpin, aT);
    }
	
	class SpriterObject extends SpriterBone {
        private _spriter;
        private _sprite;
        constructor(aSpriter: Spriter, aSprite: Phaser.Sprite);
        getSprite(): Phaser.Sprite;
        setOn(aOn: boolean): void;
        setKey(aAnimation: Animation, aTimelineId: number, aKeyId: number): void;
        update(aParent: SpatialInfo): void;
        updateSprite(): void;
    }

    interface ISpriterGroupListener {
        onAnimationFinished(aSpriterGroup: SpriterGroup): void;
    }


    class SpriterGroup extends Phaser.Group {
        private _spriter;
        private _textureKey;
        private _entityName;
        private _currentAnimationName;
        private _animation;
        private _animationSpeed;
        private _bones;
        private _objects;
        private _time;
        private _nextTime;
        private _keyIndex;
        private _root;
        private _pause;
        private _finished;
        private _listener;
        constructor(aGame: Phaser.Game, aSpriter: Spriter, aTextureKey: string, aEntityName: string, aAnimation?: string | number, aAnimationSpeedPercent?: number);
        setListener(aListener: ISpriterGroupListener): void;
        getAnimationCount(): number;
        getCurrentAnimationName(): string;
        setAnimationSpeedPercent(aAnimationSpeedPercent?: number): void;
        setAnimationById(aAnimationId: number): void;
        setAnimationByName(aAnimationName: string): void;
        private setAnimation(aAnimation);
        private clearTime();
        private getNextMainLineKeyTime(aTime);
        private setBones(aBones, aForce?);
        private setObjects(aObjects, aForce?);
        private loadKeys(aMainLineKeyIndex, aForce?);
        updateAnimation(): void;
        private updateCharacter();
        onLoop(): void;
        onFinished(): void;
    }
    class TimelineKey {
        setCurveType(value: Spriter.eCurveType): void;
        protected _id: number;
        protected _time: number;
        protected _spin: number;
        private _curveType;
        protected _c1: number;
        protected _c2: number;
        protected _c3: number;
        protected _c4: number;
        constructor(aId: number, aTime: number, aSpin: number);
        setCurve(aCurveType: eCurveType, aC1?: number, aC2?: number, aC3?: number, aC4?: number): void;
        getId(): number;
        getTime(): number;
        getSpin(): number;
        getCurveType(): eCurveType;
        getC1(): number;
        getC2(): number;
        getC3(): number;
        getC4(): number;
    }
	    class Entity {
        private _id;
        private _name;
        private _objectInfos;
        private _animations;
        constructor(aId: number, aName: string);
        addObjectInfo(aObjectInfo: ObjectInfo): void;
        getObjectInfoById(aId: number): ObjectInfo;
        getObjectInfoByName(aName: string): ObjectInfo;
        addAnimation(aAnimation: Animation): void;
        getAnimationById(aId: number): Animation;
        getAnimationByName(aName: string): Animation;
        getAnimationsCount(): number;
        getId(): number;
        getName(): string;
    }
	    class Folder {
        private _id;
        private _name;
        private _files;
        constructor(aId: number, aName: string);
        addFile(aFile: SpriterFile): void;
        getFileById(aId: number): SpriterFile;
        getFileByName(aName: string): SpriterFile;
        getId(): number;
        getName(): string;
    }
	    class ObjectInfo {
        private _id;
        private _type;
        private _name;
        private _width;
        private _height;
        constructor(aId: number, aName: string, aType: eObjectType, aWidth: number, aHeight: number);
        getId(): number;
        getType(): eObjectType;
        getName(): string;
        getWidth(): number;
        getHeight(): number;
    }
	    class SpriterFile {
        private _id;
        private _name;
        private _anchorX;
        private _anchorY;
        constructor(aId: number, aName: string, aAnchorX: number, aAnchorY: number);
        getId(): number;
        getName(): string;
        getAnchorX(): number;
        getAnchorY(): number;
    }    enum eAnimationLooping {
        NO_LOOPING = 0,
        LOOPING = 1,
    }
    class Animation {
        private _id;
        private _name;
        private _length;
        private _loopType;
        private _mainLineKeys;
        private _timelines;
        constructor(aId: number, aName: string, aLength: number, aLoopType: eAnimationLooping);
        getMainLineKeys(): MainLineKey[];
        addMainLineKey(aMainLineKey: MainLineKey): void;
        addTimeline(aTimeline: Timeline): void;
        getTimelineById(aId: number): Timeline;
        getTimelineByName(aName: string): Timeline;
        getId(): number;
        getName(): string;
        getLength(): number;
        getLoopType(): eAnimationLooping;
    }    enum eCurveType {
        LINEAR = 0,
        INSTANT = 1,
        QUADRATIC = 2,
        CUBIC = 3,
    }
    class CurveType {
        static getCurveTypeForName(aCurveTypeName: string): eCurveType;
    }
	    class MainLineKey {
        private _time;
        private _boneRefs;
        private _objectRefs;
        constructor(aTime: number);
        getBoneRefs(): Ref[];
        addBoneRef(aBoneRef: Ref): void;
        getObjectRefs(): Ref[];
        addObjectRef(aObjectRef: Ref): void;
        getTime(): number;
    }
	    class ObjectTimelineKey extends SpatialTimelineKey {
        private _folder;
        private _file;
        setFolderAndFile(aFolder: number, aFile: number): void;
        getFolder(): number;
        getFile(): number;
    }    enum eObjectType {
        SPRITE = 0,
        BONE = 1,
    }
    class ObjectType {
        static getObjectTypeForName(aTypeName: string): eObjectType;
    }
	    class Ref {
        id: number;
        parent: number;
        timeline: number;
        key: number;
        z: number;
        constructor(aId: number, aParent: number, aTimeline: number, aKey: number, aZ?: number);
    }
	    class SpatialInfo {
        x: number;
        y: number;
        scaleX: number;
        scaleY: number;
        pivotX: number;
        pivotY: number;
        alpha: number;
        angle: number;
    }
	    class SpatialTimelineKey extends TimelineKey {
        protected _info: SpatialInfo;
        getInfo(): SpatialInfo;
    }
	    class Timeline {
        private _id;
        private _name;
        private _type;
        private _objectRef;
        private _keys;
        constructor(aId: number, aName: string, aType?: eObjectType, aObjectRef?: number);
        getKeys(): TimelineKey[];
        addKey(aKey: TimelineKey): void;
        getId(): number;
        getName(): string;
        getType(): eObjectType;
        getObjectRef(): number;
    }
	    class NodeListXml implements ISpriterNodeList {
        private _file;
        private _nodeList;
        constructor(aSpriterXml: SpriterXml, aNodeList: NodeList);
        length(): number;
        processed(): void;
        getChildNodes(aIndex: number, aElementName: string): ISpriterNodeList;
        getFolder(aIndex: number): Folder;
        getFile(aIndex: number): SpriterFile;
        getEntity(aIndex: number): Entity;
        getObjectInfo(aIndex: number): ObjectInfo;
        getAnimation(aIndex: number): Animation;
        getMainLineKey(aIndex: number): MainLineKey;
        getRef(aIndex: number): Ref;
        getTimeline(aIndex: number): Timeline;
        getTimelineKey(aIndex: number, aSpriter: Spriter): TimelineKey;
    }
	    interface ISpriterNodeList {
        length(): number;
        processed(): void;
        getChildNodes(aIndex: number, aElementName: string): ISpriterNodeList;
        getFolder(aIndex: number): Folder;
        getFile(aIndex: number): SpriterFile;
        getEntity(aIndex: number): Entity;
        getObjectInfo(aIndex: number): ObjectInfo;
        getAnimation(aIndex: number): Animation;
        getMainLineKey(aIndex: number): MainLineKey;
        getRef(aIndex: number): Ref;
        getTimeline(aIndex: number): Timeline;
        getTimelineKey(aIndex: number, aSpriter: Spriter): TimelineKey;
    }
	    class LoaderSpriterFile {
        protected _minimized: boolean;
        private _minDefs;
        private _minDefsStack;
        getNodes(aNodeName: string): ISpriterNodeList;
        processed(): void;
        protected setMinimized(aMinimized: boolean, aMinDefs?: any): void;
        protected getFileNameWithoutExtension(aPath: string): string;
        protected translateElementName(aElementName: string): string;
        protected translateChildElementName(aElementName: string): string;
        protected translateAttributeName(aAttributeName: string): string;
        protected setMinDefsToElementName(aTagName: string): void;
        protected popMinDefsStack(): void;
    }
	    class SpriterXml extends LoaderSpriterFile {
        private _xml;
        constructor(aXmlData: XMLDocument, aMinDefs?: any);
        private parseInt(aElement, aAttributeName, aDefaultValue?);
        protected parseFloat(aElement: Element, aAttributeName: string, aDefaultValue?: number): number;
        protected parseString(aElement: Element, aAttributeName: string, aDefaultValue?: string): string;
        getNodes(aNodeName: string): ISpriterNodeList;
        getNodesForElement(aElement: Element, aNodeName: string): ISpriterNodeList;
        getFolder(aElement: Element): Folder;
        getFile(aElement: Element): SpriterFile;
        getEntity(aElement: Element): Entity;
        getObjectInfo(aElement: Element, aIndex: number): ObjectInfo;
        getAnimation(aElement: Element): Animation;
        getMainLineKey(aElement: Element): MainLineKey;
        getRef(aElement: Element): Ref;
        getTimeline(aElement: Element): Timeline;
        getTimelineKey(aElement: Element, aIndex: number, aSpriter: Spriter): TimelineKey;
    }
	    class Loader {
        private _spriter;
        load(aFile: Spriter.LoaderSpriterFile): Spriter;
        private loadFolders(aSpriter, aFolders);
        private loadFiles(aFolder, aFiles);
        private loadEntities(aSpriter, aEntities);
        private loadBones(aEntity, aBones);
        private loadAnimations(aEntity, aAnimations);
        private loadMainlineKeys(aAnimation, aMainLineKeys);
        private loadTimelines(aAnimation, aTimelines);
        private loadTimelineKeys(aTimeline, aKeys);
    }
}
