/// <reference path="lib/ts/phaser.d.ts" />
/// <reference path="lib/ts/spriterRuntime.d.ts" />
var SpriterObj = (function () {
    function SpriterObj(textureKey, entityName, initialAnimation, game, data, x, y) {
        this.speed = new Phaser.Point(0, 0);
        this.currentAnim = "";
        this.inverted = 1;
        this.turnDelay = 0;
        this.lastTurn = -1000;
        this.spriterGroup = new Spriter.SpriterGroup(game, data, textureKey, entityName, 0, 100);
        this.spriterGroup.position.setTo(x, y);
        this.spriterGroup.setAnimationByName(initialAnimation);
        game.world.add(this.spriterGroup);
    }
    SpriterObj.prototype.distanceTo = function (obj) {
        return Math.sqrt(Math.pow(this.getPos().x - obj.getPos().x, 2) + Math.pow(this.getPos().y - obj.getPos().y, 2));
    };
    SpriterObj.prototype.update = function (tick, map, layer) {
        if (tick > this.lastTurn + this.turnDelay) {
            if (this.speed.x > 0.01) {
                this.spriterGroup.scale.x = -1 * this.inverted;
                this.lastTurn = tick;
            }
            else if (this.speed.x < -0.01) {
                this.spriterGroup.scale.x = this.inverted;
                this.lastTurn = tick;
            }
        }
        this.calcAnimation(tick);
        //8 = max Speed ever
        if (this.speed.getMagnitude() > 8) {
            this.speed.normalize();
            this.speed.multiply(8, 8);
        }
        var newX = Math.round(this.getPos().x + this.speed.x);
        var newY = Math.round(this.getPos().y + this.speed.y);
        //noinspection JSSuspiciousNameCombination
        if (Math.round(this.getPos().x) != newX || Math.round(this.getPos().y) != newY) {
            if (SpriterObj.isBlocked(newX, newY, map, layer)) {
                if (SpriterObj.isBlocked(this.getPos().x, newY, map, layer)) {
                    if (!SpriterObj.isBlocked(newX, this.getPos().y, map, layer)) {
                        this.setPos(newX, this.getPos().y);
                    }
                    else {
                        this.speed.set(0, 0);
                    }
                }
                else {
                    this.setPos(this.getPos().x, newY);
                }
            }
            else {
                this.setPos(newX, newY);
            }
        }
    };
    SpriterObj.isBlocked = function (x, y, map, layer) {
        y -= 30;
        var tile = map.getTile(layer.getTileX(x), layer.getTileY(y)).index;
        return (tile > 11) && (tile != 27);
    };
    SpriterObj.prototype.calcAnimation = function (tick) {
    };
    SpriterObj.prototype.setPos = function (x, y) {
        this.spriterGroup.position.setTo(x, y);
    };
    SpriterObj.prototype.getPos = function () {
        return this.spriterGroup.position;
    };
    SpriterObj.prototype.updateAnimation = function () {
        this.spriterGroup.updateAnimation();
    };
    SpriterObj.prototype.setAnimation = function (name) {
        if (this.currentAnim != name) {
            this.currentAnim = name;
            this.spriterGroup.setAnimationByName(this.currentAnim);
        }
    };
    return SpriterObj;
})();
//# sourceMappingURL=spriterObj.js.map