/// <reference path="lib/ts/phaser.d.ts" />
/// <reference path="lib/ts/spriterRuntime.d.ts" />

class SpriterObj{
    spriterGroup:Spriter.SpriterGroup;

    speed: Phaser.Point = new Phaser.Point(0, 0);
    currentAnim:string = "";
    inverted:number = 1;
    turnDelay:number = 0;

    constructor(textureKey:string, entityName:string, initialAnimation:string, game:Phaser.Game, data:any, x:number, y:number) {
        this.spriterGroup = new Spriter.SpriterGroup(game, data, textureKey, entityName, 0, 100);
        this.spriterGroup.position.setTo(x, y);
        this.spriterGroup.setAnimationByName(initialAnimation);
        game.world.add(this.spriterGroup);
    }


    distanceTo(obj: SpriterObj) {
        return Math.sqrt( Math.pow(this.getPos().x - obj.getPos().x, 2) + Math.pow(this.getPos().y - obj.getPos().y, 2));
    }

    lastTurn:number = -1000;

    update(tick:number, map:Phaser.Tilemap, layer:Phaser.TilemapLayer ) {
        if (tick > this.lastTurn + this.turnDelay) {
            if (this.speed.x > 0.01) {
                this.spriterGroup.scale.x = -1 * this.inverted;
                this.lastTurn = tick;
            } else if (this.speed.x < -0.01) {
                this.spriterGroup.scale.x = this.inverted;
                this.lastTurn = tick;
            }
        }
        this.calcAnimation(tick);

        //8 = max Speed ever
        if (this.speed.getMagnitude() > 8) {
            this.speed.normalize();
            this.speed.multiply(8, 8);
        }

        var newX = Math.round(this.getPos().x + this.speed.x);
        var newY = Math.round(this.getPos().y + this.speed.y);

        //noinspection JSSuspiciousNameCombination
        if ( Math.round(this.getPos().x) != newX || Math.round(this.getPos().y) != newY) {

            if (SpriterObj.isBlocked(newX, newY, map, layer)) {
                if (SpriterObj.isBlocked(this.getPos().x, newY, map, layer)) {
                    if (!SpriterObj.isBlocked(newX, this.getPos().y, map, layer)) {
                        this.setPos(newX, this.getPos().y);
                    } else {
                        this.speed.set(0, 0);
                    }
                } else {
                    this.setPos(this.getPos().x, newY);
                }
            } else {
                this.setPos(newX, newY);
            }
        }
    }

    static isBlocked(x:number, y:number, map:Phaser.Tilemap, layer:Phaser.TilemapLayer) {
        y -= 30;
        var tile = map.getTile(layer.getTileX(x), layer.getTileY(y)).index;
        return (tile > 11) && (tile != 27);
    }


    calcAnimation(tick:number) {
    }

    setPos(x:number, y:number) {
        this.spriterGroup.position.setTo(x, y);
    }

    getPos() {
        return this.spriterGroup.position;
    }

    updateAnimation() {
        this.spriterGroup.updateAnimation();
    }

    setAnimation(name:string) {
        if (this.currentAnim != name) {
            this.currentAnim = name;
            this.spriterGroup.setAnimationByName(this.currentAnim);
        }
    }
}